@@ -23,6 +23,7 @@ SOCKETWAITTIME = 1
 KEEPALIVETIMEOUT = 25
 RECONNECTTIMEOUT = 5
 TIDBLOCKTIME = 90
+MAXIMUMSENDTIME = 3
 
 class DcsConnection:
     __serial__ = None
@@ -117,6 +118,8 @@ class DcsConnection:
         result = self.__dcsConnect__()
         if not result:
             logging.warning(str(id(self)) + " Failed to connect " + str(self.__serial__ ) + "@" + str(self.__serverAddress__) + ":" + str(self.__serverPort__) + ". Check encryption key!")
+        else:
+            logging.info(str(id(self)) + " Successfully connected " + str(self.__serial__ ) + "@" + str(self.__serverAddress__) + ":" + str(self.__serverPort__) + ".")
         self.__keepConnected__ = True
         return result
 
@@ -191,6 +194,7 @@ class DcsConnection:
 
             if(wait == True):
                 wait = False
+                logging.warning(str(id(self)) + " Waiting for socket to recover")
                 time.sleep(SOCKETWAITTIME)
 
             if self.__receiveBuffer__ != None and len(self.__receiveBuffer__)>=32:			
@@ -208,25 +212,39 @@ class DcsConnection:
                     message = {
                         "tid": rec_tid,
                         "type": rec_msgtype,
-                        "payload": rec_payload
+                        "payload": rec_payload,
+                        "receivetime": datetime.now()
                     }
                     self.__receivedMessageQueue__.put(message)
                     self.__lastMessageTime__ = datetime.now()
 
+                    if self.__receivedMessageQueue__.qsize() >= 2:      #todo LIMIT als konstante
+                        logging.warning(str(id(self)) + " receivedMessageQueue size exceeded. Size: " + str(self.__receivedMessageQueue__.qsize()))
+            else:
+                logging.debug(str(id(self)) + " nothing to receive.")
+                time.sleep(0.1)
+ 
     #handling massages
     def __worker__(self):
         logging.debug(str(id(self)) + " Worker running...")
-        while self.__running__ == True:
-            message = None         
+        while self.__running__ == True:        
             try:
                 message = self.__receivedMessageQueue__.get_nowait()
             except Exception:
-                pass
+                message = None
+                time.sleep(0.1)
 
             if(message):
                 logging.debug(str(id(self)) + " Handling Message: " + str(message))
                 try:
-                    self.__handlemessage__(message["type"], message["payload"], message["tid"])
+                    difference = datetime.now() - message["receivetime"]
+                    if(difference.total_seconds() < 5):     #Todo Konstante definieren
+                        self.__handlemessage__(message["type"], message["payload"], message["tid"])
+                        processingtime = datetime.now() - message["receivetime"]
+                        if(processingtime.total_seconds() > 5):     #Todo Konstante definieren
+                            logging.warning(str(id(self)) + " processing time too log: " + str(difference.total_seconds()))
+                    else:
+                        logging.warning(str(id(self)) + " message to old. Rejecting message. Age: " + str(difference.total_seconds()))
                 except Exception as e:
                     logging.error(str(id(self)) + " Error handling message. " + str(e))
 
@@ -269,6 +287,8 @@ class DcsConnection:
 
         if(msgtype == 6):
             logging.info(str(id(self)) + " TID: " + str(tid) + "; Received Calc Log Data (for MSG-Type 2).")
+            # if payload != None:
+            #     decryptedpayload = self.__decryptPayload__(payload)
             nack=True
             try:
                 self.loadlogdata()
@@ -451,6 +471,7 @@ class DcsConnection:
         self.__messageLock__.release()
 
     def __send__(self, payload, msgtype, tid, isresponce=False, nack=False):
+        startsendtime = datetime.now()
         if self.__socket__ == None or self.__socket__._closed:
             errmsg = str(id(self)) + "Cannot send. No open Socket."
             logging.error(errmsg)
@@ -477,6 +498,11 @@ class DcsConnection:
                     if(not self.__waitForAck__(tid)): 
                         logging.warning(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " no ACK received.")
                         return False
+
+                difference = datetime.now() - startsendtime
+                if(difference.total_seconds() > MAXIMUMSENDTIME):
+                    logging.warning(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " sending time to hight! " + str(difference.total_seconds()))
+
                 logging.debug(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " successfull send.")
                 return True
             else:
@@ -488,53 +514,48 @@ class DcsConnection:
     def __registermessage__(self, tid, msgtype):
         if msgtype == 9:
             return True;
-        self.__messageLock__.acquire()
-        self.__cleanSentrequests__()
-        try:
-            self.__sentRequests__[tid]
-            if self.__sentRequests__[tid] != None:
-                logging.error("[" + self.__serial__ + "] tid " + str(tid) + " already in use")
-                self.__messageLock__.release()
-                return False
-        except Exception:
-            pass
+        with self.__messageLock__:
+            self.__cleanSentrequests__()
+            try:
+                self.__sentRequests__[tid]
+                if self.__sentRequests__[tid] != None:
+                    logging.error("[" + self.__serial__ + "] tid " + str(tid) + " already in use")
+                    return False
+            except Exception:
+                pass
 
-        event = threading.Event()
-        self.__sentRequests__[tid] = {
-            "timestamp": datetime.now(),
-            "event": event,
-            "state": "NEW"
-        }
-        self.__messageLock__.release()
-        return True
+            event = threading.Event()
+            self.__sentRequests__[tid] = {
+                "timestamp": datetime.now(),
+                "event": event,
+                "state": "NEW"
+            }
+            return True
 
     def __waitForAck__(self, tid):
-        self.__messageLock__.acquire()
-        try:
-            if(self.__sentRequests__[tid]["state"] != "ACK"):
-                self.__sentRequests__[tid]["state"] = "SEND"
-                self.__sentRequests__[tid]["timestamp"] = datetime.now()
-                event = self.__sentRequests__[tid]["event"]
-            else:
-                event = None
-        except Exception as e:
-            logging.warning(str(id(self)) + " TID: " + str(tid) + " MSG not registrated. " + str(e))
-            self.__messageLock__.release()
-            return False
-        self.__messageLock__.release()
+        with self.__messageLock__:
+            try:
+                if(self.__sentRequests__[tid]["state"] != "ACK"):
+                    self.__sentRequests__[tid]["state"] = "SEND"
+                    self.__sentRequests__[tid]["timestamp"] = datetime.now()
+                    event = self.__sentRequests__[tid]["event"]
+                else:
+                    event = None
+            except Exception as e:
+                logging.warning(str(id(self)) + " TID: " + str(tid) + " MSG not registrated. " + str(e))
+                return False
 
         if(event):
             event.wait(ACKTIMEOUT)
 
         response = False
-        self.__messageLock__.acquire()
-        try:
-            if(self.__sentRequests__[tid]["state"] == "ACK"):
-                response = True
-            del self.__sentRequests__[tid]
-        except Exception as e:
-            logging.warning(str(id(self)) + " TID: " + str(tid) + " MSG not found. " + str(e))
-        self.__messageLock__.release()
+        with self.__messageLock__:
+            try:
+                if(self.__sentRequests__[tid]["state"] == "ACK"):
+                    response = True
+                del self.__sentRequests__[tid]
+            except Exception as e:
+                logging.warning(str(id(self)) + " TID: " + str(tid) + " MSG not found. " + str(e))
         return response
 
     def __wakeUpWaitingMessages__(self):
@@ -723,13 +744,13 @@ class DcsConnection:
         if(self.__device__ == None):
             logging.warning(str(id(self)) + " device not found. getting dummy logData")
             t = datetime.now()
-            self.__logData__ = {"i_power": t.second * 100}
+            self.__logData__ = {"i_power": t.second * 100, "time": str(t)}
         else:
             self.__logData__ = self.__device__.getlogdata()
 
     def clearlogdata(self):
         if(self.__device__ == None):
-            logging.warning(str(id(self)) + " device not found. nothing to clean.")
+            logging.debug(str(id(self)) + " device not found. nothing to clean.")
         else:
             try:
                 self.__device__.clearlog()
