@@ -77,11 +77,14 @@ class ModbusConnection:     #Singleton
             try:
                 self.__modbusConnection__ = self.__modbusClient__.connect()
                 if(self.__modbusConnection__ != True):
-                    logging.error("[ModbusConnection] Cannot connect modbus.")
+                    logging.error("[ModbusConnection] Cannot connect modbus. Check modbus module.")
             except Exception as e:
                 logging.error("[ModbusConnection] Unknown modbus connection error: " + str(e))
         return self.__modbusConnection__
 
+    def checkmodbusconnection(self):
+        return self.__modbusConnection__
+
     def writeregister(self, node, registerId, value):
         result = None
         with self.__mutex__:
@@ -136,7 +139,7 @@ class ModbusConnection:     #Singleton
                         self.__increaseModbusErrorCounter__(node)
                         time.sleep(BUSRECOVERYTIME)
                 else:
-                    logging.warning("[ModbusConnection] Cannot read from node " + str(node) + ".")
+                    logging.debug("[ModbusConnection] Cannot read from node " + str(node) + ". Modbus-Socket not connected.")
             return None
 
     def readinputregisters(self, node, startAddress = 1000, registersToRead = 1):
